@@ -27,6 +27,7 @@ import org.gcube.portlets.user.td.resourceswidget.client.utils.UtilsGXT3;
 import org.gcube.portlets.user.td.widgetcommonevent.client.event.SessionExpiredEvent;
 import org.gcube.portlets.user.td.widgetcommonevent.client.type.SessionExpiredType;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
+import org.gcube.portlets.user.td.widgetcommonevent.shared.mime.MimeTypeSupport;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.uriresolver.ApplicationType;
 import org.gcube.portlets.user.td.wizardwidget.client.WizardListener;
 
@@ -108,7 +109,7 @@ public class ResourcesPanel extends FramedPanel {
 	private MenuItem saveItem;
 	private MenuItem removeItem;
 
-	private RemoveResourceSession removeResourceSession;	
+	private RemoveResourceSession removeResourceSession;
 	private SaveResourceSession saveResourceSession;
 
 	public ResourcesPanel(ResourcesDialog parent, TRId trId, EventBus eventBus) {
@@ -231,12 +232,18 @@ public class ResourcesPanel extends FramedPanel {
 						ResourceTD resource = value.getResourceTD();
 						String resourceData = "";
 						if (resource instanceof InternalURITD) {
-							InternalURITD internalURITD = (InternalURITD) resource;
-							resourceData = "<tr>"
-									+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>File Id: </b></td>"
-									+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
-									+ SafeHtmlUtils.htmlEscape(internalURITD
-											.getFileId()) + "</td>" + "</tr>";
+							/*
+							 * InternalURITD internalURITD = (InternalURITD)
+							 * resource; if (internalURITD.getStringValue() !=
+							 * null &&
+							 * !internalURITD.getStringValue().isEmpty()) {
+							 * resourceData = "<tr>" +
+							 * "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>File Id: </b></td>"
+							 * +
+							 * "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+							 * + SafeHtmlUtils .htmlEscape(internalURITD
+							 * .getStringValue()) + "</td>" + "</tr>"; }
+							 */
 
 						} else {
 							if (resource instanceof StringResourceTD) {
@@ -290,35 +297,34 @@ public class ResourcesPanel extends FramedPanel {
 
 		ColumnConfig<ResourceTDDescriptor, ResourceTDType> typeColumn = new ColumnConfig<ResourceTDDescriptor, ResourceTDType>(
 				props.resourceType(), 30, "Type");
-	
+
 		ResourceTDTypeButtonCell button = new ResourceTDTypeButtonCell();
 		button.addSelectHandler(new SelectHandler() {
 
 			@Override
 			public void onSelect(SelectEvent event) {
 				Log.debug("Button  Pressed");
-				
+
 				Context c = event.getContext();
 				int rowIndex = c.getIndex();
-				int columnIndex=c.getColumn();
-				
-				Element el=grid.getView().getCell(rowIndex, columnIndex);
-				
-				NativeEvent contextEvent = Document.get().createMouseEvent(BrowserEvents.CONTEXTMENU, true, true,0, 0, 0,
-				        el.getAbsoluteLeft(), el.getAbsoluteTop(), false, false, false, false,
-				        NativeEvent.BUTTON_RIGHT, null);
-				
-				//NativeEvent contextEvent = Document.get().createContextMenuEvent();		
+				int columnIndex = c.getColumn();
+
+				Element el = grid.getView().getCell(rowIndex, columnIndex);
+
+				NativeEvent contextEvent = Document.get().createMouseEvent(
+						BrowserEvents.CONTEXTMENU, true, true, 0, 0, 0,
+						el.getAbsoluteLeft(), el.getAbsoluteTop(), false,
+						false, false, false, NativeEvent.BUTTON_RIGHT, null);
+
+				// NativeEvent contextEvent =
+				// Document.get().createContextMenuEvent();
 				el.dispatchEvent(contextEvent);
-				//DomEvent
+				// DomEvent
 			}
 		});
 
 		typeColumn.setCell(button);
 
-	
-		
-		
 		List<ColumnConfig<ResourceTDDescriptor, ?>> l = new ArrayList<ColumnConfig<ResourceTDDescriptor, ?>>();
 		l.add(expander);
 		l.add(nameCol);
@@ -358,9 +364,7 @@ public class ResourcesPanel extends FramedPanel {
 					}
 				});
 			}
-			
-			
-			
+
 		};
 
 		grid.setLoader(loader);
@@ -487,6 +491,19 @@ public class ResourcesPanel extends FramedPanel {
 					contextMenu.add(removeItem);
 					grid.setContextMenu(contextMenu);
 					break;
+				case GENERIC_FILE:
+					contextMenu.clear();
+					openItem.setIcon(ResourceBundle.INSTANCE.file());
+					contextMenu.add(openItem);
+					contextMenu.add(saveItem);
+					contextMenu.add(removeItem);
+					grid.setContextMenu(contextMenu);
+					break;
+				case GENERIC_TABLE:
+					contextMenu.clear();
+					contextMenu.add(removeItem);
+					grid.setContextMenu(contextMenu);
+					break;
 				default:
 					contextMenu.clear();
 					contextMenu.add(removeItem);
@@ -504,23 +521,28 @@ public class ResourcesPanel extends FramedPanel {
 	protected void requestSave(ResourceTDDescriptor resourceTDDescriptor) {
 		switch (resourceTDDescriptor.getResourceType()) {
 		case CHART:
-			requestSaveResource(resourceTDDescriptor, ".jpg", "image/jpeg");
+			requestSaveResource(resourceTDDescriptor, MimeTypeSupport._jpg);
 			break;
 		case CODELIST:
 			break;
 		case CSV:
-			requestSaveResource(resourceTDDescriptor, ".csv", "text/csv");
+			requestSaveResource(resourceTDDescriptor, MimeTypeSupport._csv);
 			break;
 		case GUESSER:
 			break;
 		case JSON:
-			requestSaveResource(resourceTDDescriptor, ".json",
-					"application/json");
+			requestSaveResource(resourceTDDescriptor, MimeTypeSupport._json);
 			break;
 		case MAP:
 			break;
 		case SDMX:
 			break;
+		case GENERIC_FILE:
+			requestSaveResource(resourceTDDescriptor, MimeTypeSupport._unknow);
+			break;
+		case GENERIC_TABLE:
+			// requestSaveResource(resourceTDDescriptor, MimeTypeSupport._csv);
+			break;
 		default:
 			break;
 
@@ -529,29 +551,15 @@ public class ResourcesPanel extends FramedPanel {
 	}
 
 	protected void requestSaveResource(
-			ResourceTDDescriptor resourceTDDescriptor, String extension,
-			String mimeType) {
-		
-		InternalURITD internalURITD;
-		ResourceTD resource = resourceTDDescriptor.getResourceTD();
-		if (resource instanceof InternalURITD) {
-			internalURITD=(InternalURITD)resource;
-		} else {
-			UtilsGXT3.alert("Attention", "This resource does not have valid internal URI!");
-			Log.debug("Attention, this resource does not have valid Internal URI!");
-			return;
-		}
-		Log.debug("InteranlURI: "+internalURITD);
-		
+			ResourceTDDescriptor resourceTDDescriptor, MimeTypeSupport mime) {
+
 		saveResourceSession = new SaveResourceSession();
-		saveResourceSession.setMimeType(mimeType);
-		saveResourceSession.setExtension(extension);
-		saveResourceSession.setFileName(resourceTDDescriptor.getName()
-				+ extension);
+		saveResourceSession.setResourceTDDescriptor(resourceTDDescriptor);
+		saveResourceSession.setMime(mime);
+		saveResourceSession.setFileName(resourceTDDescriptor.getName());
 		saveResourceSession.setFileDescription(resourceTDDescriptor
 				.getDescription());
-		saveResourceSession.setUri(internalURITD.getUri());
-		
+
 		GWT.runAsync(new RunAsyncCallback() {
 
 			public void onSuccess() {
@@ -561,9 +569,9 @@ public class ResourcesPanel extends FramedPanel {
 
 				saveResourceWizard.addListener(new WizardListener() {
 
-					public void failed(String title, String message, String details,
-							Throwable throwable) {
-						Log.debug(title + ", " + message+" "+details);
+					public void failed(String title, String message,
+							String details, Throwable throwable) {
+						Log.debug(title + ", " + message + " " + details);
 					}
 
 					public void completed(TRId id) {
@@ -647,6 +655,11 @@ public class ResourcesPanel extends FramedPanel {
 			break;
 		case SDMX:
 			break;
+		case GENERIC_FILE:
+			requestOpenGenericFile(resourceTDDescriptor);
+			break;
+		case GENERIC_TABLE:
+			break;
 		default:
 			break;
 
@@ -655,9 +668,10 @@ public class ResourcesPanel extends FramedPanel {
 	}
 
 	private void repquestOpenChart(ResourceTDDescriptor resourceTDDescriptor) {
-		ChartViewerDialog chartDialog=new ChartViewerDialog(resourceTDDescriptor, trId, eventBus);
+		ChartViewerDialog chartDialog = new ChartViewerDialog(
+				resourceTDDescriptor, trId, eventBus);
 		chartDialog.show();
-		
+
 	}
 
 	protected void requestOpenMap(
@@ -693,20 +707,86 @@ public class ResourcesPanel extends FramedPanel {
 
 		} else {
 			if (resource instanceof InternalURITD) {
-				/*
-				 * Log.error(
-				 * "Error with resource: is internal uri resource no uri present"
-				 * ); UtilsGXT3.alert("Error with resource",
-				 * "Error is string resource no uri present");
-				 */
+				InternalURITD internalURITD = (InternalURITD) resource;
+				UriResolverSession uriResolverSession = new UriResolverSession(
+						internalURITD.getId(), ApplicationType.GIS);
+
+				TDGWTServiceAsync.INSTANCE.getUriFromResolver(
+						uriResolverSession, new AsyncCallback<String>() {
+
+							public void onFailure(Throwable caught) {
+								if (caught instanceof TDGWTSessionExpiredException) {
+									eventBus.fireEvent(new SessionExpiredEvent(
+											SessionExpiredType.EXPIREDONSERVER));
+								} else {
+									Log.error("Error with uri resolver: "
+											+ caught.getLocalizedMessage());
+									UtilsGXT3
+											.alert("Error",
+													"Error retrieving uri from resolver");
+								}
+							}
+
+							public void onSuccess(String link) {
+								Log.debug("Retrieved link: " + link);
+								Window.open(link,
+										resourceTDDescriptor.getName(), "");
+							}
+
+						});
+
 			} else {
 				if (resource instanceof TableResourceTD) {
-					/*
-					 * Log.error(
-					 * "Error with resource: is table resource no uri present");
-					 * UtilsGXT3.alert("Error with resource",
-					 * "Error is table resource no uri present");
-					 */
+
+				} else {
+					Log.error("Error with resource: no valid resource");
+					UtilsGXT3.alert("Error with resource",
+							"Error no valid InternalUri");
+
+				}
+
+			}
+		}
+	}
+
+	protected void requestOpenGenericFile(
+			final ResourceTDDescriptor resourceTDDescriptor) {
+		ResourceTD resource = resourceTDDescriptor.getResourceTD();
+		if (resource instanceof InternalURITD) {
+			InternalURITD genericFileResourceTD = (InternalURITD) resource;
+			UriResolverSession uriResolverSession = new UriResolverSession(
+					genericFileResourceTD.getId(), ApplicationType.SMP_ID,
+					resourceTDDescriptor.getName(),
+					genericFileResourceTD.getMimeType());
+
+			TDGWTServiceAsync.INSTANCE.getUriFromResolver(uriResolverSession,
+					new AsyncCallback<String>() {
+
+						public void onFailure(Throwable caught) {
+							if (caught instanceof TDGWTSessionExpiredException) {
+								eventBus.fireEvent(new SessionExpiredEvent(
+										SessionExpiredType.EXPIREDONSERVER));
+							} else {
+								Log.error("Error with uri resolver: "
+										+ caught.getLocalizedMessage());
+								UtilsGXT3.alert("Error",
+										"Error retrieving uri from resolver");
+							}
+						}
+
+						public void onSuccess(String link) {
+							Log.debug("Retrieved link: " + link);
+							Window.open(link, resourceTDDescriptor.getName(),
+									"");
+						}
+
+					});
+		} else {
+			if (resource instanceof InternalURITD) {
+
+			} else {
+				if (resource instanceof TableResourceTD) {
+
 				} else {
 					Log.error("Error with resource: no valid resource");
 					UtilsGXT3.alert("Error with resource",
@@ -737,7 +817,7 @@ public class ResourcesPanel extends FramedPanel {
 								Log.error("Error Retrieving Resources: "
 										+ caught.getLocalizedMessage());
 								UtilsGXT3.alert("Error retrieving resources",
-										"Error retrieving resources");
+										caught.getLocalizedMessage());
 							}
 						}
 						callback.onFailure(caught);
